Package: SmCCNet
Title: Sparse Multiple Canonical Correlation Network Analysis Tool
        ('SmCCNet')
Version: 2.0.7
Date: 2026-04-30
Authors@R: c(person(given = "Abhinav",
                    family = "Pundir",
                    role =  "cre",
                    email="abhinav.pundir@ucdenver.edu"),
            person(given = "Weixuan",
                    family = "Liu",
                    role = "aut",
                    email = "weixuan.liu@cuanschutz.edu"),
             person(given = "Yonghua",
                    family = "Zhuang",
                    role = "aut"),
             person(given = c("W.", "Jenny"),
                    family = "Shi",
                    role = "aut"),
             person(given = "Thao",
                    family = "Vu",
                    role = "aut"),
             person(given = "Iain",
                    family = "Konigsberg",
                    role = "aut"),
             person(given = "Katherine",
                    family = "Pratte",
                    role = "aut"),
             person(given = "Laura",
                    family = "Saba",
                    role = "aut"),
             person(given = "Katerina",
                    family = "Kechris",
                    role = "aut"))
Description: 
   A canonical correlation based framework ('SmCCNet') designed for the construction of phenotype-specific multi-omics networks. This framework adeptly integrates single or multiple omics data types along with a quantitative or binary phenotype of interest. It offers a streamlined setup process that can be tailored manually or configured automatically, ensuring a flexible and user-friendly experience. Methods are described in Shi et al. (2019) "Unsupervised discovery of phenotype-specific multi-omics networks" <doi:10.1093/bioinformatics/btz226>.
URL: https://github.com/KechrisLab/SmCCNet,
        https://kechrislab.github.io/SmCCNet/,
        https://liux4283.github.io/SmCCNet/
Depends: R (>= 3.5)
Imports: EnvStats, future, pROC, spls, Matrix, pbapply, igraph,
        magrittr, rlist, furrr, purrr, pracma
License: GPL-3
Encoding: UTF-8
LazyData: true
biocViews: Network
RoxygenNote: 7.3.3
NeedsCompilation: no
VignetteBuilder: knitr
Suggests: knitr, rmarkdown, bookdown, testthat (>= 3.0.0), dplyr,
        reshape2, shadowtext, tidyverse, parallel, mltools, caret,
Config/testthat/edition: 3
Packaged: 2026-04-30 20:36:50 UTC; pundira
Author: Abhinav Pundir [cre],
  Weixuan Liu [aut],
  Yonghua Zhuang [aut],
  W. Jenny Shi [aut],
  Thao Vu [aut],
  Iain Konigsberg [aut],
  Katherine Pratte [aut],
  Laura Saba [aut],
  Katerina Kechris [aut]
Maintainer: Abhinav Pundir <abhinav.pundir@ucdenver.edu>
Repository: CRAN
Date/Publication: 2026-05-01 18:50:07 UTC
Built: R 4.5.2; ; 2026-05-02 01:34:10 UTC; unix
